@@ -11,45 +11,33 @@
 #' @example inst/examples/ex_afttest.R
 #' @export
 print.afttest <- function(x, ...) {
-  if (!inherits(x,"afttest")) stop("Must be afttest class")
-  
+  cat("\n\tAFT Goodness-of-Fit Test\n\n")
   cat("Call: \n")
   print(x$call)
   
-  cat("\n p-values: \n")
-  p_valueTAB <- data.frame(t(c(x$p_value, x$p_std_value)))
+  cat("\n--- Null Hypothesis (H0) ---\n")
+  if (x$testType %in% c("omnibus", "omni")) {
+    cat(paste0("The assumed semiparametric AFT model fits the data adequately. \n"))
+  } else if (x$testType == "link") {
+    cat(paste0("The relationship between covariates and the log survival time is \n"))
+    cat(paste0("correctly specified. \n"))
+  } else if (x$testType %in% c("covForm", "form")) {
+    cov_name <- ifelse(!is.null(x$covTested), x$covTested, "selected covariate")
+    cat(paste0("The functional form for covariate '", cov_name, "' is correctly specified.\n"))
+    # cat(paste0("The relationship between the log survival time and the specific \n"))
+    # cat(paste0("covariate chosen by the covTested argument is correctly specified. \n"))
+  }
+  
+  cat("\n--- p-values ---\n")
+  p_vals <- c(x$p_value, x$p_std_value)
+  p_vals_fmt <- format.pval(p_vals, digits = 3, eps = 0.001)
+  p_valueTAB <- data.frame(t(p_vals_fmt))
   rownames(p_valueTAB) <- ""
   colnames(p_valueTAB) <- c("unstandardized", "standardized")
   print(p_valueTAB)
   
-  if (x$testType == "omnibus") {
-    cat(paste0("\n H0: The assumed semiparametric AFT model fits the data adequately. \n"))
-  } else if (x$testType == "link") {
-    cat(paste0("\n H0: The relationship between covariates and the log survival time "))
-    cat(paste0("\n     is correctly specified. \n"))
-  } else if (x$testType == "covform") {
-    cat(paste0("\n H0: The relationship between the log survival time and the specific "))
-    cat(paste0("\n     covariate chosen by the cov.tested argument is correctly specified. \n"))
-  }
-  
-  # if (x$estMethod=="ls"){
-  #   cat(paste0("\n Coefficients (estimated by aftgee::aftgee):"))
-  # } else if (x$estMethod=="rr"){
-  #   cat(paste0("\n Coefficients (estimated by aftgee::aftsrr):"))
-  # }
-  # coefTAB <- data.frame(t(-x$beta))
-  # rownames(coefTAB) <- ""
-  # colnames(coefTAB) <- x$names[-c(1:2)]
-  # print(coefTAB)
-  
   invisible(x)
 }
-# print.afttest <- function(x, ...) {
-#   if (!inherits(x,"afttest")) stop("Must be afttest class")
-#   
-#   print.default(x)
-#   invisible(x)
-# }
 
 ##############################################################################
 ## summary
@@ -64,36 +52,49 @@ print.afttest <- function(x, ...) {
 #' @example inst/examples/ex_afttest.R
 #' @export
 summary.afttest <- function(object, ...) {
-  if (!inherits(object,"afttest")) stop("Must be afttest class")
-  
-  cat("Call: \n")
+  cat("\n\tAFT Goodness-of-Fit Test Summary\n\n")
+  cat("Call:\n")
   print(object$call)
   
-  cat("\n p-values: \n")
-  p.valueTAB <- data.frame(t(c(object$p_value, object$p_std_value)))
+  cat("\n--- Null Hypothesis (H0) ---\n")
+  if (object$testType %in% c("omnibus", "omni")) {
+    cat(paste0("The assumed semiparametric AFT model fits the data adequately. \n"))
+  } else if (object$testType == "link") {
+    cat(paste0("The relationship between covariates and the log survival time is \n"))
+    cat(paste0("correctly specified. \n"))
+  } else if (object$testType %in% c("covForm", "form")) {
+    cov_name <- ifelse(!is.null(object$covTested), object$covTested, "selected covariate")
+    cat(paste0("The functional form for covariate '", cov_name, "' is correctly specified.\n"))
+    # cat(paste0("The relationship between the log survival time and the specific \n"))
+    # cat(paste0("covariate chosen by the covTested argument is correctly specified. \n"))
+  }
+  
+  cat("\n--- Simulation Parameters ---\n")
+  cat(paste("Test Type:       ", object$testType, "\n"))
+  cat(paste("Bootstrap:       ", ifelse(object$linApprox, "Linear Approximation", "Standard Resampling"), "\n"))
+  cat(paste("Resampling Paths:", object$npath, "\n"))
+  cat(paste("Random Seed:     ", object$seed, "\n"))
+  
+  cat("\n--- p-values ---\n")
+  p_vals <- c(object$p_value, object$p_std_value)
+  p_vals_fmt <- format.pval(p_vals, digits = 3, eps = 0.001)
+  
+  p.valueTAB <- data.frame(t(p_vals_fmt))
   rownames(p.valueTAB) <- ""
   colnames(p.valueTAB) <- c("unstandardized", "standardized")
   print(p.valueTAB)
   
-  if (object$testType == "omnibus") {
-    cat(paste0("\n H0: The assumed semiparametric AFT model fits the data adequately. \n"))
-  } else if (object$testType == "link") {
-    cat(paste0("\n H0: The relationship between covariates and the log survival time "))
-    cat(paste0("\n     is correctly specified. \n"))
-  } else if (object$testType == "covform") {
-    cat(paste0("\n H0: The relationship between the log survival time and the specific "))
-    cat(paste0("\n     covariate chosen by the cov.tested argument is correctly specified. \n"))
+  method_name <- ifelse(object$estMethod == "ls", "Least Squares", "Rank-Based")
+  cat(paste0("\n--- Coefficients (Method: ", method_name, ") ---"))
+  if (object$estMethod=="ls"){
+    cat(paste0("\n Coefficients (estimated by aftgee::aftgee):"))
+  } else if (object$estMethod=="rr"){
+    cat(paste0("\n Coefficients (estimated by aftgee::aftsrr):"))
   }
-  
-  # if (object$estMethod=="ls"){
-  #   cat(paste0("\n Coefficients (estimated by aftgee::aftgee):"))
-  # } else if (object$estMethod=="rr"){
-  #   cat(paste0("\n Coefficients (estimated by aftgee::aftsrr):"))
-  # }
-  # coefTAB <- data.frame(t(-object$beta))
-  # rownames(coefTAB) <- ""
-  # colnames(coefTAB) <- object$names[-c(1:2)]
-  # print(coefTAB)
+  coefTAB <- data.frame(t(object$beta))
+  rownames(coefTAB) <- ""
+  colnames(coefTAB) <- object$names[-c(1:2)]
+  print(coefTAB)
   
   invisible(object)
 }
@@ -119,7 +120,7 @@ summary.afttest <- function(object, ...) {
 #'    some quantiles of x, which are used in weight, are plotted for graphs, 
 #'    i.e. 0\%, 10\%, 25\%, 40\%, 50\%, 60\%, 75\%, 90\%, and 100\% are used.}
 #'    \item{link}{an x of the link function test is the form of n by 1 matrix}
-#'    \item{covform}{an x of the functional form test is the form of n by 1 matrix}
+#'    \item{covForm}{an x of the functional form test is the form of n by 1 matrix}
 #' }
 #'    See the documentation of \pkg{ggplot2} and \pkg{gridExtra} for details.
 #' 
@@ -127,33 +128,37 @@ summary.afttest <- function(object, ...) {
 #' @importFrom gridExtra grid.arrange
 #' @importFrom stats quantile
 #' 
-#' @example inst/examples/ex_plot.afttest.R
+#' @example inst/examples/ex_afttest.R
 #' @export
 plot.afttest <- function(x, npath = 50, std = TRUE, quantile = NULL, ...){
-  
-  # class
-  if (!inherits(x,"afttest")) return(warning("Must be afttest class"))
-  # npathsave
-  if ((x$npathsave<1)) return(warning("afttest is conduced with npathsave=0"))
   # eqType
   eqType <- x$eqType
+  
   # testType
   testType <- x$testType
+  
   # std
   if (isTRUE(std)) {
     std <- "std"
   } else if (!std %in% c(T, F)) {
     return(warning("std needs to be logical."))
   }
+  
+  # npathsave
+  if (is.null(x$npathsave) || x$npathsave == 0) {
+    stop("Plot cannot be created: 'afttest' was conducted with npathsave=0. No paths were stored.")
+  }
+  
   # npath
-  if (!is.numeric(npath) || !(length(npath)==1)){
-    return(warning("npath needs to be a positive integer."))
-  } else {
-    if (!is.numeric(npath)) {
-      npath <- 50
-    } else {
-      npath <- min(npath,x$npathsave)
-    }
+  if (!is.numeric(npath) || !(length(npath)==1 || npath <= 0)){
+    return(warning("Argument 'npath' must be a single positive integer."))
+  }
+  npath <- as.integer(npath)
+  if (npath > x$npathsave) {
+    # Ideally, warn the user that we are capping their request
+    warning(sprintf("Requested npath (%d) exceeds stored paths (%d). Plotting all %d available paths.", 
+                    npath, x$npathsave, x$npathsave))
+    npath <- x$npathsave
   }
   
   stdTypeQuote <- ifelse(std=="std","standardized","unstandardized")
@@ -217,27 +222,27 @@ plot.afttest <- function(x, npath = 50, std = TRUE, quantile = NULL, ...){
         y_breaks <- round(seq(y_breaksMIN, y_breaksMAX, length.out = 5),1)
         y_labels <- format(y_breaks, nsmall = 1)
         Figure_k <-
-          ggplot() +
-          geom_step(data=DF_app,aes(x=resid,y=app,group=group),colour="grey",alpha=0.5) +
-          geom_step(data=DF_obs,aes(x=resid,y=obs),colour="tomato",lwd=0.25) +
-          labs(y = NULL) + labs(x = NULL) +
-          ggtitle(paste0("Omnibus test ","(",stdTypeQuote,")")) +
+          ggplot2::ggplot() +
+          ggplot2::geom_step(data=DF_app,ggplot2::aes(x=resid,y=app,group=group),colour="grey",alpha=0.5) +
+          ggplot2::geom_step(data=DF_obs,ggplot2::aes(x=resid,y=obs),colour="tomato",lwd=0.25) +
+          ggplot2::labs(y = NULL) + ggplot2::labs(x = NULL) +
+          ggplot2::ggtitle(paste0("Omnibus test ","(",stdTypeQuote,")")) +
           # ggtitle(paste0("Omnibus test ","(",stdTypeQuote,")"," with ",names(Q[k]), " percentile for z")) +
-          scale_y_continuous(breaks = y_breaks, labels = y_labels) +
-          theme(plot.title=element_text(hjust=0.5),
+          ggplot2::scale_y_continuous(breaks = y_breaks, labels = y_labels) +
+          ggplot2::theme(plot.title=element_text(hjust=0.5),
                 plot.margin = rep(unit(0,"null"),4),
                 panel.spacing = unit(0,"null"))
       } else {
         y_breaks <- round(seq(y_breaksMIN, y_breaksMAX, length.out = 3),1)
         y_labels <- format(y_breaks, nsmall = 1)
         Figure_k <-
-          ggplot() +
-          geom_step(data=DF_app,aes(x=resid,y=app,group=group),colour="grey",alpha=0.5) +
-          geom_step(data=DF_obs,aes(x=resid,y=obs),colour="tomato",lwd=0.25) +
-          labs(y = NULL) + labs(x = NULL) + 
-          scale_y_continuous(breaks = y_breaks, labels = y_labels) +
-          geom_label(aes(x=-Inf,y=Inf),label=paste0(names(Q[k])),fill="darkgrey",label.size=NA,size=3,hjust=-0.0,vjust=1.0) +
-          theme(plot.margin = rep(unit(0,"null"),4),
+          ggplot2::ggplot() +
+          ggplot2::geom_step(data=DF_app,ggplot2::aes(x=resid,y=app,group=group),colour="grey",alpha=0.5) +
+          ggplot2::geom_step(data=DF_obs,ggplot2::aes(x=resid,y=obs),colour="tomato",lwd=0.25) +
+          ggplot2::labs(y = NULL) + ggplot2::labs(x = NULL) + 
+          ggplot2::scale_y_continuous(breaks = y_breaks, labels = y_labels) +
+          ggplot2::geom_label(aes(x=-Inf,y=Inf),label=paste0(names(Q[k])),fill="darkgrey",label.size=NA,size=3,hjust=-0.0,vjust=1.0) +
+          ggplot2::theme(plot.margin = rep(unit(0,"null"),4),
                 panel.spacing = unit(0,"null")) 
       }
       Figure[[k]] <- Figure_k
@@ -293,7 +298,7 @@ plot.afttest <- function(x, npath = 50, std = TRUE, quantile = NULL, ...){
     
     return(Figure)
     
-  } else if(testType=="covform"){
+  } else if(testType=="covForm"){
     resid <- c(NA)
     app <- c(NA)
     obs <- c(NA)
